--------------------------------------------------------------------------------
-- Carnage Contest AI (Artificial Intelligence) Script
-- COMBO AI v1.0
--------------------------------------------------------------------------------

ai={}
-- the list of non-dead enemies
players={}

-- GHOST AI comment
-- how to shoot: by upper or lower trajectory
-- upper is effective in surface (big chance to avoid terrain if bazooka is primary weapon)
-- looks really cool (like boomerang)
-- bad in cave (hits the ceiling)
-- sometimes misses because angle can't be ajusted precizely)
-- lower is more straightforward
ai.C=1


-- Weapons which fire projectiles that aren't affected by gravity and wind
ai.directweapons = { "Gamma Gun", "Sniper", "Plasma Blaster", "Minigun", "Plasma Gun", "Laser", "Machine Gun","Pistol", "Paintgun" }

-- weapons that have a global effect. the player just has to press the fire key ("shoot and forget")
ai.shootandforgetweapons = { "Flood", "Sance", "Scales of Justicia", "Pandemic", "Winter", "Armageddon", "Arrow Rain", "Anti-Gravity", "Hellfire", "Quake", "Santa", "Minerain" }

-- items that increase the health of the player but don't end the round
ai.healthitems = { "Apple", "Cookie" }

ai.toolstack = {}
ai.taskstack = {}

ai.enemydamage = 0
ai.enemykills = 0
ai.allydamage = 0
ai.allykills = 0

ai.idealmode = nil
ai.idealsettings = {}

ai.roundstart = true

ai.attempts = 0
-- Initialization Function (called once at each start of AI turn)
function ai_init()
	if(ai.roundstart==true) then
		randomseed(getframesleft()*12345)
		ai.taskstack = {0,1,2,3,4}
		ai.roundstart=false
		ai_next()
	end
	if(ai.attacktype==0) then		-- Prepare
		ai_print("PREPARE MODE")
		ai.waittimer=50
		if(getplayerstate(0,state_poisoned)==1) then
			table.insert(ai.toolstack, "Antidote")
		end
--		if(getplayerstate(0,state_confused)==1 or getplayerstate(0,state_poisoned)==1 or getplayerstate(0,state_frozen)==1) then
--			table.insert(ai.toolstack, "Effect Neutralizer")
--		end
		
		if(getplayerstate(0,state_invisible)==0) then
			table.insert(ai.toolstack, "Stealth Ninja")
		end
		
		if(getplayerhealth(0)<75) then
			for i=1,#ai.healthitems do
				table.insert(ai.toolstack, ai.healthitems[i])
			end
		end
		
	end
	if(ai.attacktype==1) then		-- Gamma AI
		ai_print("GAMMA AI")
		-- select weapon
		selected=false
		ai.noclip=false
		ai.movetimer=0
		ai.waittimer=50
		for i=1,#ai.directweapons do
			if(ai_weapon(ai.directweapons[i])==1) then
				ai_print("selected weapon "..ai.directweapons[i])
				selected=true
				if i==1 then
					ai.noclip=true
				end
				break
			end
		end
		if(selected==false) then
			ai_print("Weapon selection failed")
			--ai_print("Skipping turn")
			--ai_weapon("End Turn")
			ai.attacktype=2
		else
			ai.movetimer=1
			ai.power=200
			players=playertable(3,0)
			
			success=false
			for i=1,#players*5 do
				ai.target=math.random(1,table.maxn(players))
				ai.targetx=getplayerx(players[ai.target])
				ai.targety=getplayery(players[ai.target])
				ai.aim=math.ceil(math.deg(math.atan2(ai.targetx-getplayerx(0),-ai.targety+getplayery(0))))
				if ai.aim<0 then
					ai.dir=0
				else
					ai.dir=1
				end
				
				-- if the player is frozen, check wheather the player need to turn round. If the player needs to, forget the target and go to the next one.
				-- if the player is not frozen, continue
				if((getplayerstate(0,state_frozen)==1 and (ai.dir==0 and getplayerdirection(0)==-1) or (ai.dir==1 and getplayerdirection(0)==1)) or getplayerstate(0,state_frozen)==0) then
					-- if the gun goes through terrain or the laser would hit a player
					if(ai.noclip==true or laserattack()==1) then
						ai_print("Number of enemies: "..table.maxn(players))
						ai_print("Target is player number "..ai.target)
						ai_print("His relative coordinates: "..ai.targetx-getplayerx(0).."|"..ai.targety-getplayery(0))
						ai_print("Set aim to: "..ai.aim)
						success=true
						break
					end
				end
			end
			if(success==false) then
				ai_print("Target selection failed")
				--ai_print("Skipping turn")
				--ai_weapon("End Turn")
				ai_next()
			else	
				
			end
		end
	end
	if(ai.attacktype==2) then		-- Ghost AI
		ai_print("GHOST AI")
		local cont = false
		-- select Ghost Rocket
		ai.wpn = ai_weapon("Ghost Rocket")
		if(ai.wpn==1) then
			ai.weapon="Ghost Rocket"
			ai_print("Selected weapon: Ghost Rocket")
			cont = true
		else
			ai_print("No Ghost Rocket available.")
			ai.wpn = ai_weapon("Bazooka")
			if(ai.wpn==1) then
				ai_print("Selected weapon: Bazooka")
				ai.weapon="Bazooka"
				cont = true
			else
				ai_print("No Bazooka available.")

				ai_next()
			end
		end

		if(cont==true) then
			ai.movetimer=1
			ai.power=100
			players=playertable(3,0)
			ai_print("Gravity is: "..getgravity())
			ai_print("Wind is: "..getwind())
			-- coordinates translation for wind compensation
			-- wind is the same as gravity in cc physics, so we choose coordinates so that
			-- acceleration vector (gravity + wind) goes by y axis. 
			ai.phi=math.atan2(getwind(),getgravity())
			ai_print("rotate coordinates by: "..math.deg(ai.phi))
			-- if wind is zero, phi=0, if wind blows right and equals gravity, phi=pi/4 etc.		
			ai.accel=math.sqrt(getgravity()^2+getwind()^2)
			ai_print("Eq. gravity is: "..ai.accel)
			-- value of total acceleration

			ai_print("Evaluation of targets")
			ai_print("Number of enemies: "..table.maxn(players))
			local targetlocked = nil
			for i=1,table.maxn(players),1 do
				ai_print("Player number: "..i)
				ai.targetx=getplayerx(players[i])-getplayerx(0)
				ai.targety=getplayery(players[i])-getplayery(0)
				-- translate coordinates
				ai.targetxtr=ai.targetx*math.cos(ai.phi)-ai.targety*math.sin(ai.phi)
				ai.targetytr=ai.targety*math.cos(ai.phi)+ai.targetx*math.sin(ai.phi)
				ai_print("His relative coordinates: "..ai.targetx.."|"..ai.targety)
				ai_print("Translated coordinates are: "..ai.targetxtr.."|"..ai.targetytr)
				ai.D=1-ai.accel/100*(ai.accel*(ai.targetxtr)^2/100-2*(ai.targetytr))
				if ai.D>0 then
					local str = "Of 2 different trajectories"
					if(ai.weapon=="Ghost Rocket") then
						ai.C = math.random(0,1)
					elseif(ai.weapon=="Bazooka") then
						ai.C = 1
					end
					
					targetlocked=false
					
					local a=math.random(0,1)
					local start,stop,step
					if(a==0) then
						start=0; stop=1; step=1
					else
						start=1; stop=0; step=-1
					end					
					
					for i=start,stop,step do
						ai.C = i
						if ai.C>0 then
							ai.aim=math.pi/2-math.atan(100/ai.accel/math.abs(ai.targetxtr)*(1+math.sqrt(ai.D)))
							ai_print("checking upper trajectory ...")
						else
							ai.aim=math.pi/2-math.atan(100/ai.accel/math.abs(ai.targetxtr)*(1-math.sqrt(ai.D)))
							ai_print("checking lower trajectory ...")
						end				
						if ai.targetxtr<0 then
							ai.aim=-ai.aim
						end
						-- translate them back by substracting phi
						ai.aim=mat_round(math.deg(ai.aim-ai.phi))
						
						if ai.aim>180 then
							ai.aim=ai.aim-360
						elseif ai.aim<-180 then
							ai.aim=ai.aim+360
						end
						if ai.aim<0 then
							ai.dir=0
						else
							ai.dir=1
						end
						ai_print("Set aim to: "..ai.aim)
						
						if((getplayerstate(0,state_frozen)==1 and (ai.dir==0 and getplayerdirection(0)==-1) or (ai.dir==1 and getplayerdirection(0)==1)) or getplayerstate(0,state_frozen)==0) then
							ai_print("Player direction OK.")
							if(ai.weapon=="Bazooka") then
								if(bazookaattack()==1) then
									ai_print("Bazooka will hurt enemy.")
									targetlocked = true	
									break
								else
									ai_print("Bazooka will hit terrain only")
									targetlocked = false
								end
							elseif(ai.weapon=="Ghost Rocket") then
								break
							end
						else
							ai_print("Player direction bad. I'd must turn round for that shot but can't.")
						end
					end		
					
					if(targetlocked==true) then

						break
					end
				else
					ai_print("can't hit this enemy: not enough power")
					targetlocked = false
				end
			end
			if(targetlocked==true) then
				
			else
				ai_print("I can't hit any enemy.")
				ai_next()
			end
		end
	end
	if(ai.attacktype==3) then		-- Shoot and Forget
		---> SHOOT AND FORGET
		ai_print("SHOOT AND FORGET")
		local selected=false
		for i=1,#ai.shootandforgetweapons do
			--rnd = math.random(1,#ai.shootandforgetweapons)
			rnd = i
			wpn = ai_weapon(ai.shootandforgetweapons[rnd])
			if(wpn==1) then
				selected=true
				ai.weapon=ai.shootandforgetweapons[rnd]
				if(ai.weapon=="Flood") then
					ai_print("Checking Flood ...")
					local allies=playertable(2,0)
					table.insert(allies,0)
					local enemies=playertable(3,0)
					local floodapplicable = true
					local alliesdrown=0
					local enemiesdrown=0
					for i=1,#allies do
						if(getplayery(allies[i])>=getwatery()-138) then
							alliesdrown=alliesdrown+1
						end
					end

					for i=1,#enemies do
						if(getplayery(enemies[i])>=getwatery()-138) then
							enemiesdrown = enemiesdrown+1
						end
					end
					ai_print("Of "..#allies.." allies, "..alliesdrown.." will drown.")
					ai_print("Of "..#enemies.." enemies, "..enemiesdrown.." will drown.")
					
					-- if it kills at least 1 enemy and no allies or kills all enemies but not all allies, the flood is applicable
					if((enemiesdrown>0 and alliesdrown==0) or (enemiesdrown==#enemies and alliesdrown<#allies)) then
						floodapplicable=true
					else
						floodapplicable=false
					end
					
					if(floodapplicable==true) then
						ai_print("Flood applicable!")
						selected = true
						break
					else
						ai_print("Flood not applicable!")
						selected = false
					end
				elseif(ai.weapon=="Scales of Justicia") then
					ai_print("checking Scales of Justicia")
					friends=playertable(2,0)
					table.insert(friends,0)
					local ownteamhealth=0
					local totalhealth = 0
					local players = 0
					local teammembers = {0, 0, 0, 0, 0, 0, 0, 0}
					for p=1,#friends do
						ownteamhealth = ownteamhealth + getplayerhealth(friends[p])
					end

					for p=1,#playertable() do
						local player = playertable()[p]
					
						if(getplayerhealth(player)>=1) then
							totalhealth = totalhealth+getplayerhealth(playertable()[p])
							players = players+1
							teammembers[getplayerteam(player)+1] = teammembers[getplayerteam(player)+1]+1
						end
					end

					local teams = 0
					for t=1,#teammembers do
						if(teammembers[t]>0) then
							teams = teams+1
						end
					end
					local averageteamhealth = totalhealth/teams
					ai_print("total health: "..totalhealth)
					ai_print("average team health: "..averageteamhealth)
					ai_print("health of my team: "..ownteamhealth)
					if(ownteamhealth < averageteamhealth) then
						ai_print("Scales of Justica applicable!")
						scalesapplicable = true
						break	
					else
						ai_print("Scales of Justica not applicable!")
						scalesapplicable = false
						selected = false
					end
				elseif(ai.weapon=="Pandemic") then
					ai_print("Chcking Pandemic ...")
					local poisoned = 0
					local enemies = playertable(3,0)
					
					for i=1,#enemies do
						if(getplayerstate(enemies[i],state_poisoned)==1) then
							poisoned = poisoned+1
						end
					end
					ai_print("Of "..#enemies.." enemies, "..poisoned.." are already poisoned.")
					if(poisoned<#enemies/2) then
						ai_print("Pandemic is applicable.")
						selected = true
						break
					else
						ai_print("Pandemic is not applicable.")
						selected = false
					end				
				elseif(ai.weapon=="Winter") then
					ai_print("Chcking Winter ...")
					local frozen = 0
					local enemies = playertable(3,0)
					
					for i=1,#enemies do
						if(getplayerstate(enemies[i],state_frozen)==1) then
							frozen = frozen+1
						end
					end
					ai_print("Of "..#enemies.." enemies, "..frozen.." are frozen")
					if(frozen<#enemies/2) then
						ai_print("Winter is applicable!")
						selected = true
						break
					else
						ai_print("Winter isn't applicable!")
						selected = false
					end							
				elseif(ai.weapon=="Sance") then
					ai_print("Checking Sance ...")
					local teammates = playertable(1,1)
					local allies = playertable(2,0)
					local enemies = playertable(3,0)
					
					table.insert(teammates,0)
					
					for i=1,#teammates do
						id = teammates[i]
						
						local enemydamage = 0
						local enemykills = 0
						local allydamage = 0
						local allykills = 0
						
						if(getplayerhealth(id)<=0) then
							for e=1,#enemies do
								if(ai_playerdistance(teammates[i],enemies[e])<=100) then
									local damage = (100-ai_playerdistance(teammates[i],enemies[e]))*1.50
									enemydamage = enemydamage + damage
									if(damage >= getplayerhealth(enemies[e])) then
										enemykills = enemykills + 1
									end
								end
							end
							for a=1,#allies do
								if(ai_playerdistance(teammates[i],allies[a])<=100) then
									local damage = (100-ai_playerdistance(teammates[i],allies[a]))*1.50
									allydamage = allydamage + damage
									if(damage >= getplayerhealth(allies[a])) then
										allykills = allykills + 1
									end
								end
							end
						end
						
						ai_print("guessed enemy damage: "..enemydamage)
						ai_print("guessed enemy kills: "..enemykills)
						ai_print("guessed ally damage: "..allydamage)
						ai_print("guessed ally kills: "..allykills)
						if ((enemydamage > 50 or enemykills>0) and (allydamage < 30 and allykills==0) or (enemykills==#enemies and allykills<#allies)) then
							ai_print("Sance is applicable!")
							selected=true
							break
						else
							ai_print("The Sance is not applicable.")
							selected=false
						end
					end
				else
					break
				end
			end
		end
		if(selected==false) then
			ai_print("No catastrophe available.")
			ai_next()
			ai.waittimer=50
		end	
	end
	if(ai.attacktype==4) then		-- Zombie AI
		ai_print("ZOMBIE AI")
		if(getplayerstate(0,state_confused)==0 and getplayerstate(0,state_frozen)==0) then
			ai.raft=false
			ai.suicide1=0
			ai.suicide2=0
			randomseed(getframesleft()*5050316)
			ai.dir=0
			players=playertable(3,0)
			ai.bestplayer=0;
			ai.bestx=9999;
			ai.lx=getplayerx(0)
			for i=1,#players,1 do
				if (math.abs(getplayerx(players[i])-ai.lx)<ai.bestx) or (ai.bestplayer==0) then
					ai.bestplayer=players[i]
					ai.bestx=math.abs(getplayerx(players[i])-ai.lx)
				end
			end
			if (getplayerx(ai.bestplayer)-ai.lx)>0 then ai.dir=1 end
			ai.lx=666
			
			-- select weapon
			ai_zombie_select()
			
			ai.targetx=getplayerx(ai.bestplayer)
			ai.targety=getplayery(ai.bestplayer)
			ai.aim=math.ceil(math.deg(math.atan2(ai.targetx-getplayerx(0),-ai.targety+getplayery(0))))
		else
			ai_print("... skipped (confused/frozen)")
			ai_next()
		end
	end
	if(ai.attacktype==5) then		-- end turn
		ai_print("PASSIVE MODE")
		ai.weapon=nil
	end
end

-- Movement Function (called each frame, pre attack)
function ai_move()
	if(ai.attacktype==0) then
		local wpn

		
		if(ai.waittimer<=0) then
			ai_next()
		else
			if(#ai.toolstack>0) then
				if(ai.weapon~=ai.toolstack[1]) then
					ai.weapon = ai.toolstack[1]
					wpn=ai_weapon(ai.weapon)
					if(wpn==0) then
						table.remove(ai.toolstack,1)
					end
				end
			end
			ai.waittimer=ai.waittimer-1
		end
		
		if ai.waittimer<=0 then
			ai_attack()
			table.remove(ai.toolstack,1)
			ai.waittimer=75
			
			if(#ai.toolstack==0) then
				ai.waittimer=0
				ai_next()
			end
		end
	end
	if(ai.attacktype==1) then
		ai.targetx=getplayerx(players[ai.target])
		ai.targety=getplayery(players[ai.target])
		ai.aim=math.ceil(math.deg(math.atan2(ai.targetx-getplayerx(0),-ai.targety+getplayery(0))))
		if(ai.waittimer>0) then
			ai.waittimer=ai.waittimer-1
		end
		if ai.movetimer>0 then
			if ai.dir==0 and getplayerdirection(0)==1 then
				if(getplayerstate(0,state_confused)==0) then
					ai_left()
				else
					ai_right()
				end
			elseif ai.dir==1 and getplayerdirection(0)==-1 then
				if(getplayerstate(0,state_confused)==0) then
					ai_right()
				else
					ai_left()
				end
			end
			ai.movetimer=ai.movetimer-1
		elseif ai.aim~=getplayerrotation(0) then
			if (ai.aim-getplayerrotation(0))*(2*ai.dir-1)>0 then
				ai_down()
			else
				ai_up()
			end
		elseif ai.power>0 and ai.waittimer<=0 then
			ai_print("new aim: "..ai.aim)
			ai_attack()
			ai.waittimer=75
			ai.power=ai.power-1
		end
	end
	if(ai.attacktype==2) then
		if(ai.wpn~=0) then
			if ai.movetimer>0 then
				if ai.dir==0 and getplayerdirection(0)==1 then
					if(getplayerstate(0,state_confused)==0) then
						ai_left()
					else
						ai_right()
					end
				elseif ai.dir==1 and getplayerdirection(0)==-1 then
					if(getplayerstate(0,state_confused)==0) then
						ai_right()
					else
						ai_left()
					end
				end
				ai.movetimer=ai.movetimer-1
			elseif ai.aim~=getplayerrotation(0) then
				if (ai.aim-getplayerrotation(0))*(2*ai.dir-1)>0 then
					ai_down()
				else
					ai_up()
				end
			elseif ai.power>0 then
				ai_attack()
				ai.power=ai.power-1
			end
		else
			ai_attack()
		end	
	end
	if(ai.attacktype==3) then
		if(ai.waittimer>0) then
			ai.waittimer=ai.waittimer-1
		else
			ai_attack()
		end
	end
	if(ai.attacktype==4) then
		if(getplayerstate(0,state_confused)==0 and getplayerstate(0,state_frozen)==0) then
			if (math.abs(getplayerx(ai.bestplayer)-getplayerx(0))<16) and ((getplayery(ai.bestplayer)-getplayery(0))<10) and ((getplayery(ai.bestplayer)-getplayery(0))>(-28)) then
				if (getplayerx(ai.bestplayer)-ai.lx)>0 then ai_right() else ai_left() end
				ai.targetx=getplayerx(ai.bestplayer)
				ai.targety=getplayery(ai.bestplayer)
				ai.aim=math.ceil(math.deg(math.atan2(ai.targetx-getplayerx(0),-ai.targety+getplayery(0))))
				ai_zombie_select()

				if ai.aim~=getplayerrotation(0) then
					if (ai.aim-getplayerrotation(0))*(2*ai.dir-1)>0 then
						ai_down()
					else
						ai_up()
					end
				else		
					ai_attack()
				end
			else
				if ai.lx==getplayerx(0) then
					ai_backjump()
					ai.suicide1=ai.suicide1+1
				else
					if (ai.suicide1>0) then ai.suicide1=ai.suicide1-1 end
				end
				if (getplayery(0)>=(getwatery()-55)) then
					local wpn=0
					if(ai.raft==false) then
						wpn = ai_weapon("Raft")
						if(wpn==1) then
							ai.raft = true
						else
							ai_next()
						end
					elseif(ai.raft==true) then
						if ai.dir==0 then
							ai_left()
						else
							ai_right()
						end
					end
				else
					ai.aim=math.ceil(math.deg(math.atan2(ai.targetx-getplayerx(0),-ai.targety+getplayery(0))))
					if ai.aim~=getplayerrotation(0) then
						if (ai.aim-getplayerrotation(0))*(2*ai.dir-1)>0 then
							ai_down()
						else
							ai_up()
						end
					end			
				
					if ai.dir==0 then
						ai_left()
					else
						ai_right()
					end
				end
			end
			ai.ldir=ai.dir
			ai.lx=getplayerx(0)
			if (getplayerx(ai.bestplayer)-ai.lx)>20 then ai.dir=1
			elseif (getplayerx(ai.bestplayer)-ai.lx)<-20 then ai.dir=0
			end
			if (ai.ldir~=ai.dir) then ai.suicide2=ai.suicide2+1 end
			if (ai.suicide1>20) or (ai.suicide2>5) then
				ai.weapon=nil
				ai.bestplayer=0
				ai.suicide1=0
				ai.suicide2=0
				
				ai_next()
			end
		else
			ai_next()
		end
	end
	if(ai.attacktype==5) then
		if(ai.weapon==nil) then
			if(getplayerstate(0,state_poisoned)==1 or getplayerhealth(0)<=50) then
				ai.wpn=ai_weapon("Self-Healing")
				if(ai.wpn==1) then
					ai.weapon="Self-Healing"
					ai.waittimer=50
				end
			end
		end
		
		if(ai.weapon==nil) then
			ai.wpn=ai_weapon("End Turn")
			if(ai.wpn==1) then
				ai.weapon="End Turn"
				ai.waittimer=50
			end
		end
		
		if(ai.waittimer>0 and ai.weapon~=nil) then
			ai.waittimer=ai.waittimer-1
		elseif(ai.wpn==1) then
			ai_attack()
		end
	end
end
-- Backing Function (called each frame, post attack)
function ai_back()
	ai.roundstart = true
	ai.attacktype = 0
	ai.weapon = nil
end

--------------------------------------------------------------------------------
-- AI Helper Functions
-- Attention: These functions are also used by orig. AI weapon Lua scripts
-- !!! CHECK ALL DEPENDENCIES WHEN CHANGING !!!
--------------------------------------------------------------------------------

tracers = {}

function laserattack()								-- Attack
	if ai.aim<0 then
		ai.dir=0
	else
		ai.dir=1
	end
	-- Attack
	id=1
	tracers[id]={}
	-- Ignore collision with current player at beginning
	tracers[id].ignore=playercurrent()
	-- Set initial position of projectile
	tracers[id].x=getplayerx(0)+(7*ai.dir)-math.sin(math.rad(ai.aim))*5.0
	tracers[id].y=getplayery(0)+3+math.cos(math.rad(ai.aim))*5.0
	-- Set speed of projectile
	tracers[id].sx=math.sin(math.rad(ai.aim))*15.0
	tracers[id].sy=-math.cos(math.rad(ai.aim))*15.0
	-- Timer
	tracers[id].timer=0
	-- Initial movement
	tracers[id].x=tracers[id].x-tracers[id].sx*1.5
	tracers[id].y=tracers[id].y-tracers[id].sy*1.5
	while true do
		status = lasermove(id)
		if(status ~= -1) then
			return status
		end
	end
end
function lasermove(id)
	-- Timer (free after 10 secs)
	tracers[id].timer=tracers[id].timer+1
	if tracers[id].timer>500 then
		return 0
	end
	rot=math.deg(math.atan2(tracers[id].sx,-tracers[id].sy))
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(tracers[id].sx),math.abs(tracers[id].sy))/3)
	msubx=tracers[id].sx/msubt
	msuby=tracers[id].sy/msubt
	for i=1,msubt,1 do
		tracers[id].x=tracers[id].x+msubx
		tracers[id].y=tracers[id].y+msuby		
		-- Collision
		if collision(col3x3,tracers[id].x+math.sin(math.rad(rot))*20,tracers[id].y-math.cos(math.rad(rot))*20)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=tracers[id].ignore then
				-- Cause damage
				if playercollision()~=0 and playercollision()~=tracers[id].ignore then
					-- player hit
					if(getplayeralliance(playercollision())~=getplayeralliance(0)) then
						-- enemy hit
						return 1
					else
						-- friend hit
						return 3
					end
				elseif objectcollision()>0 then
					-- object hit
					return 2
				end
				-- terrain hit
				return 0
			end
		else
			tracers[id].ignore=0
		end
		-- Water
		if (tracers[id].y)>getwatery()+5 then
			return 0
		end
	end
	return -1
end

function bazookaattack()
	local weapon_charge = 100
	if ai.aim<0 then
		ai.dir=0
	else
		ai.dir=1
	end
	-- No more weapon switching!
	id=2
	tracers[id]={}
	-- Ignore collision with current player at beginning
	tracers[id].ignore=playercurrent()
	-- Set initial position of projectile
	tracers[id].x=getplayerx(0)+(6*ai.dir)+math.sin(math.rad(ai.aim))*10.0
	tracers[id].y=getplayery(0)+3-math.cos(math.rad(ai.aim))*10.0
	-- Initial movement
	tracers[id].sx=math.sin(math.rad(ai.aim))*20
	tracers[id].sy=-math.cos(math.rad(ai.aim))*20
	tracers[id].x=tracers[id].x-tracers[id].sx
	tracers[id].y=tracers[id].y-tracers[id].sy
	bazookamove(id)

	-- Set speed of projectile
	tracers[id].sx=math.sin(math.rad(ai.aim))*(weapon_charge/100.0)*10.0
	tracers[id].sy=-math.cos(math.rad(ai.aim))*(weapon_charge/100.0)*10.0
	
	while true do
		status = bazookamove(id)
		if(status ~= -1) then
			return status
		end
	end
end
function bazookamove(id)
	rot=math.deg(math.atan2(tracers[id].sx,-tracers[id].sy))
	-- Wind + Gravity influence on speed
	tracers[id].sx=tracers[id].sx+getwind()
	tracers[id].sy=tracers[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(tracers[id].sx),math.abs(tracers[id].sy))/3)
	msubx=tracers[id].sx/msubt
	msuby=tracers[id].sy/msubt
	for i=1,msubt,1 do
		tracers[id].x=tracers[id].x+msubx
		tracers[id].y=tracers[id].y+msuby
		-- Collision
		if collision(col3x3,tracers[id].x+math.sin(math.rad(rot))*3,tracers[id].y-math.cos(math.rad(rot))*3)==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=tracers[id].ignore then
				local enemies = playertable(3,0)
				for i=1,#enemies do
					if(mat_distance(getplayerx(enemies[i]),getplayery(enemies[i]), tracers[id].x, tracers[id].y)<=100) then
						return 1
					end
				end
				return 0
			end
		else
			tracers[id].ignore=0
		end
		-- Water
		if (tracers[id].y)>getwatery()+5 then
			-- Free projectile
			return 0
		end
	end
	return -1
end

function ai_print(text)
	print("AI: "..text)
end

function mat_round(x)
	if (x-math.floor(x))>0.5 then
		return math.ceil(x)
	else
		return math.floor(x)
	end
end

function ai_playerdistance(player1,player2)
	return mat_distance(getplayerx(player1),getplayery(player1),getplayerx(player2),getplayery(player2))
end

function mat_distance(x1,y1,x2,y2)
	return math.sqrt(math.abs(x2-x1)^2+math.abs(y2-y1)^2)
end

function ai_zombie_select()
	-- select weapon
	ai.wpn=0
	if(getplayerhealth(ai.bestplayer)>60) then
		ai.wpn = ai_weapon("Axe")
		ai.raft = false
	end
	if(ai.wpn==0) then	
		if(getplayerstate(ai.bestplayer,state_confused)==0) then
			ai.wpn = ai_weapon("Brain Punch")
			ai.raft = false
		else
			ai.wpn = ai_weapon("Uppercut")
			ai.raft = false
		end
		if(ai.wpn==0) then
			ai.wpn=ai_weapon("Uppercut")
			if(ai.wpn == 0) then
				ai.attacktype=5
				ai.raft = false
			end
		end
	end		
end

function ai_next()
	if(ai.taskstack[1]~=nil) then
		ai.attacktype = ai.taskstack[1]
		table.remove(ai.taskstack, 1)
	else
		ai.attacktype = 5
	end
	ai_init()
end